#!/usr/bin/env python

# Copyright (c) 2006 Jochen Kupperschmidt <webmaster@nwsnet.de>
# Released under the terms of the GNU General Public License
#  _                               _
# | |_ ___ _____ ___ _ _ _ ___ ___| |_
# |   | . |     | ._| | | | . |  _| . /
# |_|_|___|_|_|_|___|_____|___|_| |_|_\
#   http://homework.nwsnet.de/

"""Album Track List Generator

A module to extract artist, album title and track titles from a folder name
and the name of the files in it. That information can be used to create an
XHTML file to print out and use as CD case inlay. Useful for music bought on
the Internet.

Folder names should be in the form 'artist name - album title' and track names
in the form 'artist name - track number - track title' or 'artist name - track
title'.
You can adjust the folder and file name patterns below to fit your data.
"""

__version__ = '14-Apr-2006'


import os
import re
import sys

from simpletal import simpleTAL, simpleTALES


# Folder and file name patterns.
ALBUM_PATTERN = re.compile('^(.+?) - (.+?)$')
TRACK_PATTERN = re.compile('^.+? -( \d+? -)? (.+)\.(?:mp3|ogg)$')


class Album(list):

    def __init__(self, artist, title):
        self.artist = artist
        self.title = title

    def __str__(self):
        return "<%s, artist='%s', title='%s', %d tracks>" % (
            self.__class__.__name__, self.artist.encode('iso-8859-1'),
            self.title.encode('iso-8859-1'), len(self))

    def getDetails(self):
        head = '''\
            Artist: %(artist)s
            Title: %(title)s
            Tracks:''' % self.__dict__
        return '\n'.join([line.lstrip() for line in head.split('\n')]
            + ['  %2d. %s' % (i + 1, self[i]) for i in range(len(self))]
            ).encode('iso-8859-1')

    @classmethod
    def fromPath(cls, path):
        """Create an Album from directory and file names."""
        path = os.path.abspath(path)

        # Extract album artist and title from directory name.
        m = ALBUM_PATTERN.match(os.path.split(path)[1])
        if m is None:
            return
        album = cls(
            m.group(1).decode('iso-8859-1'), m.group(2).decode('iso-8859-1'))

        # Extract track number and title from matching file names.
        for fname in os.listdir(path):
            if not os.path.isfile(os.path.join(path, fname)):
                continue
            m = TRACK_PATTERN.match(fname)
            if m is None:
                continue
            album.append(m.group(2).decode('iso-8859-1'))

        return album


class Template(object):

    def __init__(self, fname):
        self.tpl = simpleTAL.compileXMLTemplate(open(fname))

    def expand(self, album, outputFile=sys.stdout):
        assert isinstance(album, Album)
        context = simpleTALES.Context()
        context.addGlobal('album', album)
        return self.tpl.expand(context, outputFile,
            suppressXMLDeclaration=True,
            docType='<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"'
                + ' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">')


if __name__ == '__main__':
    import sys
    if len(sys.argv) != 2:
        print 'Usage: %s <directory>' % os.path.basename(sys.argv[0])
        sys.exit(2)
    album = Album.fromPath(sys.argv[1])
    if album is None:
        print 'Could not recognize album.'
    else:
        print album.getDetails()
